@@ -8,6 +8,7 @@ import time
 import ntpath
 import pathlib
 from string import Template
+from uuid import uuid1
 
 import rpyc
 from rpyc.core.async_ import AsyncResultTimeout
@@ -36,6 +37,7 @@ class Executor(metaclass=ABCMeta):
     """
 
     def __init__(self, problem):
+        self.uuid = uuid1().hex
         self.problem = problem
 
         self.options = ConfigDictionary()
@@ -259,6 +261,7 @@ class CondorJobExecutor(RemoteExecutor):
 
     def _create_desc(self, individual):
         desc = {}
+        desc["executor_id"] = self.uuid
         desc["individual_id"] = individual.id
         desc["individual_population_id"] = individual.population_id
         desc["individual_algorithm_id"] = individual.algorithm_id
@@ -300,7 +303,7 @@ class CondorJobExecutor(RemoteExecutor):
 
                 events = []
                 cnt = 0
-                return_value = -1
+                successful_job = False
                 delay = 0.5
 
                 run = True
@@ -320,24 +323,20 @@ class CondorJobExecutor(RemoteExecutor):
 
                             if tp == "submit":
                                 # SUBMIT
-                                if "ReturnValue" in event["info"]:
-                                    return_value = event["info"]["ReturnValue"]
+                                pass
                             elif tp == "execute":
                                 # EXECUTE
-                                if "ExecuteHost" in event["info"]:
-                                    args += "ExecuteHost: {}, ".format(parse_address(event["info"]["ExecuteHost"]))
+                                args += "ExecuteHost: {}, ".format(event["execute_host"])
+                                pass
                             elif tp == "image_size":
                                 # IMAGE_SIZE
                                 pass
                             elif tp == "job_terminated":
                                 # JOB_TERMINATED
-                                if "ReturnValue" in event["info"]:
-                                    return_value = event["info"]["ReturnValue"]
+                                successful_job = event["successful"]
                                 run = False
                             elif tp == "job_held":
                                 # JOB_HELD
-                                if "ReturnValue" in event["info"]:
-                                    return_value = event["info"]["ReturnValue"]
 
                                 self.problem.logger.error("Job {}.{} is '{}' at {}".format(event["cluster"], event["proc"], event["type"], ""))
                                 run = False
@@ -364,8 +363,7 @@ class CondorJobExecutor(RemoteExecutor):
                 if (end - start) > self.problem.options["time_out"]:
                     raise TimeoutError
 
-                # successfull
-                if return_value == 0:
+                if successful_job:
                     if len(self.output_files) > 0:
                         output_files = []
                         d = datetime.datetime.now()
